import numpy as np
import matplotlib.colors
import matplotlib.pyplot as plt

class MapPlotter:

	class Axis:
		def __init__(self, field, label, format_ticks="{:.1f}", transform_func=lambda x: np.round(x,2), rotate_ticks_label=None):
			self.field = field
			self.label = label
			self.format_ticks = format_ticks
			self.transform_func = transform_func
			self.rotate_ticks_label = rotate_ticks_label

		def _apply_df(self, dataframe):
			self.all_vals = list( self.transform_func(dataframe[self.field]) )
			self.uniqued_vals = np.unique(self.all_vals)
			self.ticks = [ self.format_ticks.format(v) for v in self.uniqued_vals ]
			if self.rotate_ticks_label is None:
				self.rotate_ticks_label = 0

	def __init__(self, dataframe, axis_x, axis_y):
		# Prepare the parameters space (axis_x.uniqued_vals ⊗ axis_y.uniqued_vals)
		self.dataframe = dataframe
		self.axis_x = axis_x
		if self.axis_x.rotate_ticks_label is None:
			self.axis_x.rotate_ticks_label = 45
		self.axis_x._apply_df(dataframe)
		self.axis_y = axis_y
		self.axis_y._apply_df(dataframe)

	def plot_field(self, field, label, ax=None, groupby_agg='mean', transform_func=lambda x: x, cmap='inferno', vmin=0, vmax=None, logscale=False):

		# Retrieve and possibly transform data

		if groupby_agg is None:
			df = self.dataframe
		else:
			df = self.dataframe.groupby([
				self.axis_x.field,
				self.axis_y.field
			]).agg('mean').reset_index()

		transform_func = np.vectorize(transform_func)
		Values = list( transform_func( df[field] ) )

		# Reshape the vector Values as a matrix Mat in the parameters space (axis_x.uniqued_vals ⊗ axis_y.uniqued_vals)

		X_idx = []
		Y_idx = []
		for i in range(len(Values)):
		    X_idx += [ np.searchsorted(self.axis_x.uniqued_vals, self.axis_x.all_vals[i], side='left') ]
		    Y_idx += [ np.searchsorted(self.axis_y.uniqued_vals, self.axis_y.all_vals[i], side='left') ]

		Mat = np.empty((len(self.axis_x.uniqued_vals),len(self.axis_y.uniqued_vals)))
		Mat[X_idx,Y_idx] = Values

		# Show the matrix as a heatmap

		if ax is None:
			ax = plt.gca()
		if vmax is None:
			vmax = np.max(Mat)

		if logscale:
			norm = matplotlib.colors.LogNorm(vmin=vmin, vmax=vmax)
			im = ax.imshow(np.transpose(Mat), cmap=cmap, norm=norm)
		else:
			im = ax.imshow(np.transpose(Mat), cmap=cmap, vmin=vmin, vmax=vmax)

		ax.set_xticks(ticks=np.arange(len(self.axis_x.uniqued_vals)), labels=self.axis_x.ticks, rotation=self.axis_x.rotate_ticks_label)
		ax.set_yticks(ticks=np.arange(len(self.axis_y.uniqued_vals)), labels=self.axis_y.ticks, rotation=self.axis_y.rotate_ticks_label)
		ax.set_xlabel(self.axis_x.label)
		ax.set_ylabel(self.axis_y.label)
		cbar = plt.colorbar(im)
		cbar.ax.set_ylabel(label)